//----------------include libraries----------------//
#include <WiFi.h>
#include <WebServer.h>
#include <DMD32.h>
#include <fonts/Font_BOLD.h>
#include <FS.h>
//-------------------------------------------------//

//----------------settings panel----------------//
#define FONT Font_BOLD //Default font
#define DISPLAYS_ACROSS 1 //number of panels in width
#define DISPLAYS_DOWN 1 //number of panels in height
DMD dmd(DISPLAYS_ACROSS, DISPLAYS_DOWN);
//----------------------------------------------//

//----------------settings local wifi----------------//
const char* ssid = "SKBKIT"; //edit for your local wifi
const char* password = "skbkit2024"; //edit for your local wifi
//---------------------------------------------------//

//----------------settings web server----------------//
WebServer server(80);
IPAddress staticIP(10, 131, 170, 4);  //edit for your local ip
IPAddress gateway(10, 131, 170, 1);
IPAddress subnet(255, 255, 255, 0);
//---------------------------------------------------//

//----------------other variables----------------//
String displayText = "Привет из СКБ \"КИТ\""; //change the default label
hw_timer_t* timer = NULL;
bool panel = true;
//-----------------------------------------------//

//---Search SPI panel---//
void IRAM_ATTR triggerScan() {
  dmd.scanDisplayBySPI();
}
//----------------------//

void reader() {}

void setup() {
  Serial.begin(115200);
  dmd.selectFont(Font_BOLD);
  WiFi.begin(ssid, password);

  //---Configured WebServer---//
  if (!WiFi.config(staticIP, gateway, subnet)) {
    Serial.println("Failed to configure Static IP");
  } else {
    Serial.println("Static IP configured!");
  }
  //--------------------------//

  //---Start file system---//
  // if(!SPIFFS.begin()){
  //   Serial.print("An occurred ERROR");
  //   log_e("OCCURRED ERROR");
  //   return;
  // }
  //-----------------------//

  //---Start the WiFi---//
  while (WiFi.status() != WL_CONNECTED) {
    delay(500);
    Serial.print(".");
  }
  Serial.println("\nConnected to WiFi");
  Serial.println(WiFi.localIP());
  //--------------------//

  //---Main page for edit text---//
  server.on("/", HTTP_GET, []() {
    panel = true;
    server.send(200, "text/html",
                "<!DOCTYPE html>"
                "<html lang='ru'>"
                "<head>"
                "<meta charset='UTF-8'>"
                "<title>Panel text</title>"
                "</head>"
                "<body>"
                "<h2>Введите текст для отображения на экране:</h2>"
                "<form action='/api/text' method='post'><input name='text'><input type='submit'></form>"
                "</body>"
                "</html>");
  });
  //-----------------------------//

  //---Request to check the status of the panel---//
  server.on("/api/led", HTTP_GET, [](){
    if(panel){
      server.send(200, "application/json", "{\"state\": \"true\"}");
    }else if(!panel){
      server.send(200, "application/json", "{\"state\": \"false\"}");
    };
  });
  //----------------------------------------------//

  //---Request to change the status of the panel---//
  server.on("/api/led", HTTP_POST, [](){
    String panelState = server.arg("plain");
    Serial.println(panelState);

    if (panelState == "{\"led\": \"on\"}"){
      Serial.println("Panel on!");
      panel = true;
      //scanDisplayBySPI();
    }else if(panelState == "{\"led\": \"off\"}"){
      panel = false;
      Serial.println("Panel off!");
      SPI.end();
    }
    server.send(200, "text/html", "");
  });
  //-----------------------------------------------//

  //---A text modification request---//
  server.on("/api/text", HTTP_POST, []() {
    panel = true;
    if (server.hasArg("text")) {
      displayText = server.arg("text");
    }
    server.send(200, "text/html; charset=UTF-8",
                "<html><body><h2>Text set to:</h2><p>" + displayText + "</p></body></html>");
  });
  //---------------------------------//

  server.begin();

  //---Start the timer---//
  uint8_t cpuClock = ESP.getCpuFreqMHz();
  timer = timerBegin(0, cpuClock, true);
  timerAttachInterrupt(timer, &triggerScan, true);
  timerAlarmWrite(timer, 300, true);
  timerAlarmEnable(timer);
  //--------------------//

  dmd.clearScreen(true); //Clearing the screen
}

void loop() {
  server.handleClient();  //Processing requests from the client

  //---Screen cleaning and text output---//
  // dmd.clearScreen(true);
  // dmd.drawMarquee(displayText.c_str(), displayText.length(), (32 * DISPLAYS_ACROSS) - 1, 0);
  //-------------------------------------//

  // long start = millis();
  // long timer = start;
  // boolean ret = false;

  //---Scrolling text---//
  // while (!ret) {
  //   if ((timer + 30) < millis()) {
  //     ret = dmd.stepMarquee(-1, 0);  // Прокрутка текста
  //     timer = millis();
  //   }
  // }
  //--------------------//
}
